@@ -7,10 +7,9 @@
 //! Using the pooling instance allocator can speed up module instantiation
 //! when modules can be constrained based on configurable limits.
 
-use super::borrow_limiter;
 use super::{
     initialize_instance, initialize_vmcontext, InstanceAllocationRequest, InstanceAllocator,
-    InstanceHandle, InstantiationError, ResourceLimiter,
+    InstanceHandle, InstantiationError,
 };
 use crate::{instance::Instance, Memory, Mmap, Table, VMContext};
 use anyhow::{anyhow, bail, Context, Result};
@@ -385,19 +384,22 @@ impl InstancePool {
         instance.host_state = std::mem::replace(&mut req.host_state, Box::new(()));
         instance.wasm_data = &*req.wasm_data;
 
-        let mut limiter = req.store.and_then(|s| (*s).limiter());
+        // set_instance_memories and _tables will need the store before we can completely
+        // initialize the vmcontext.
+        if let Some(store) = req.store.as_raw() {
+            instance.set_store(store);
+        }
+
         Self::set_instance_memories(
             instance,
             self.memories.get(index),
             self.memories.max_wasm_pages,
-            borrow_limiter(&mut limiter),
         )?;
 
         Self::set_instance_tables(
             instance,
             self.tables.get(index).map(|x| x as *mut usize),
             self.tables.max_elements,
-            borrow_limiter(&mut limiter),
         )?;
 
         initialize_vmcontext(instance, req);
@@ -503,7 +505,6 @@ impl InstancePool {
         instance: &mut Instance,
         mut memories: impl Iterator<Item = *mut u8>,
         max_pages: u64,
-        mut limiter: Option<&mut dyn ResourceLimiter>,
     ) -> Result<(), InstantiationError> {
         let module = instance.module.as_ref();
 
@@ -519,12 +520,9 @@ impl InstancePool {
                 )
             };
             instance.memories.push(
-                Memory::new_static(
-                    plan,
-                    memory,
-                    commit_memory_pages,
-                    borrow_limiter(&mut limiter),
-                )
+                Memory::new_static(plan, memory, commit_memory_pages, unsafe {
+                    &mut *instance.store()
+                })
                 .map_err(InstantiationError::Resource)?,
             );
         }
@@ -538,7 +536,6 @@ impl InstancePool {
         instance: &mut Instance,
         mut tables: impl Iterator<Item = *mut usize>,
         max_elements: u32,
-        mut limiter: Option<&mut dyn ResourceLimiter>,
     ) -> Result<(), InstantiationError> {
         let module = instance.module.as_ref();
 
@@ -555,7 +552,7 @@ impl InstancePool {
 
             let table = unsafe { std::slice::from_raw_parts_mut(base, max_elements as usize) };
             instance.tables.push(
-                Table::new_static(plan, table, borrow_limiter(&mut limiter))
+                Table::new_static(plan, table, unsafe { &mut *instance.store() })
                     .map_err(InstantiationError::Resource)?,
             );
         }
@@ -907,11 +904,11 @@ impl StackPool {
 pub struct PoolingInstanceAllocator {
     strategy: PoolingAllocationStrategy,
     module_limits: ModuleLimits,
-    instance_limits: InstanceLimits,
     // This is manually drop so that the pools unmap their memory before the page fault handler drops.
     instances: mem::ManuallyDrop<InstancePool>,
     #[cfg(all(feature = "async", unix))]
     stacks: StackPool,
+    #[cfg(all(feature = "async", windows))]
     stack_size: usize,
     #[cfg(all(feature = "uffd", target_os = "linux"))]
     _fault_handler: imp::PageFaultHandler,
@@ -935,13 +932,15 @@ impl PoolingInstanceAllocator {
         #[cfg(all(feature = "uffd", target_os = "linux"))]
         let _fault_handler = imp::PageFaultHandler::new(&instances)?;
 
+        drop(stack_size); // suppress unused warnings w/o async feature
+
         Ok(Self {
             strategy,
             module_limits,
-            instance_limits,
             instances: mem::ManuallyDrop::new(instances),
             #[cfg(all(feature = "async", unix))]
             stacks: StackPool::new(&instance_limits, stack_size)?,
+            #[cfg(all(feature = "async", windows))]
             stack_size,
             #[cfg(all(feature = "uffd", target_os = "linux"))]
             _fault_handler,
@@ -1050,7 +1049,7 @@ unsafe impl InstanceAllocator for PoolingInstanceAllocator {
 #[cfg(test)]
 mod test {
     use super::*;
-    use crate::{Imports, VMSharedSignatureIndex};
+    use crate::{Imports, StorePtr, VMSharedSignatureIndex};
     use wasmtime_environ::{
         EntityRef, Global, GlobalInit, Memory, MemoryPlan, ModuleType, SignatureIndex, Table,
         TablePlan, TableStyle, WasmType,
@@ -1412,7 +1411,7 @@ mod test {
                             },
                             shared_signatures: VMSharedSignatureIndex::default().into(),
                             host_state: Box::new(()),
-                            store: None,
+                            store: StorePtr::empty(),
                             wasm_data: &[],
                         },
                     )
@@ -1436,7 +1435,7 @@ mod test {
                 },
                 shared_signatures: VMSharedSignatureIndex::default().into(),
                 host_state: Box::new(()),
-                store: None,
+                store: StorePtr::empty(),
                 wasm_data: &[],
             },
         ) {
